import urllib
from gtkeasyapp.log import Log

##FoxyTag exception
class FoxyTagException(Exception):
	##constructor
	def __init__(self, message):
		self.message = message
	
	##To string
	def __str__(self):
		return 'FoxyTagException : ' + str(message)
		
##FoxyTag class
class FoxyTag():
	##constant
	FOXYTAG_FIXED = 'F'
	FOXYTAG_MOBILE = 'M'
	FOXYTAG_GHOST = 'G'
	FOXYTAG_CANCEL = 'C'

	##constructor
	##@param tag : tag string to split
	def __init__(self, tag=''):
		#init members
		self.lat = 0.0
		self.long = 0.0
		self.kind = self.FOXYTAG_FIXED
		self.heading = 0 #in degrees
		self.opposite = False
	
		if len(tag)>0:
			#split specified tag
			(self.lat, self.long, self.kind, self.heading) = tag.split('\t')
			self.lat = float(self.lat)
			self.long = float(self.long)
			self.heading = int(self.heading)
			#remove opposite info
			if self.heading>1000:
				self.heading -= 1000
		else:
			#reset to default values
			self.lat = 0
			self.long = 0
			self.kind = self.FOXYTAG_FIXED
			self.heading = 0
			
	##To string
	def __str__(self):
		return 'FOXYTAG(lat=%.5f, long=%.5f, kind=%s, heading=%d, opposite=%s)' % (self.lat, self.long, self.kind, self.heading, self.opposite)

##Foxy class
class Foxy():
	##Constants
	__FOXYTAG_HOST = 'http://www.foxytag.com/php/dogetlab.php'
	__FOXYTAG_PORT = 80
	__FOXYTAG_USERNAME = 'test'
	__FOXYTAG_PASSWORD = ''
	__FOXYTAG_LANGUAGE = 'en'
	__FOXYTAG_CLIENT = 'speedcamalerter'
	__FOXYTAG_VERSION = '1.0.0'
	__FOXYTAG_PLATFORM = 'NokiaN810'
	
	FOXYTAG_OK_INIT = 'OK_INIT'
	FOXYTAG_OK = 'OK'
	FOXYTAG_KO = 'KO'
	FOXYTAG_KO_CMD = 'KO_CMD'
	FOXYTAG_KO_VERSION = 'KO_VERSION'
	FOXYTAG_KO_LOGIN = 'KO_LOGIN'
	FOXYTAG_ERR = 'ERR'
	FOXYTAG_INF = 'INF'
	FOXYTAG_PUB = 'PUB'
	FOXYTAG_ERR_UNKNOWN = 'UNKNOW ERROR'
	FOXYTAG_ERR_UNEXPECTED = 'UNEXPECTED ERROR'

	##Constructor
	def __init__(self):
		self.__tags = []

	##Connect to server
	##@return res : result of tag post
	##@return usize : size of uploaded data
	##@return dsize : size of downloaded data
	def connect(self):
		#init
		res = False
		usize = 0
		dsize = 0
	
		#prepare data to post
		data = urllib.urlencode({	'cmd' : 'init', 
									'username' : self.__FOXYTAG_USERNAME,
									'password' : self.__FOXYTAG_PASSWORD,
									'language' : self.__FOXYTAG_LANGUAGE,
									'client' : self.__FOXYTAG_CLIENT,
									'version' : self.__FOXYTAG_VERSION,
									'platform' : self.__FOXYTAG_PLATFORM})

		#get form
		try:
			url = self.__FOXYTAG_HOST+'?%s' % data
			handler = urllib.urlopen(url)
			lines = handler.read()
			lines = lines.split('\n')
				
			#parse results
			(result, message) = self.__parseConnectionResults(lines)
			Log.log('CONNECT result=%s message=%s' % (result, message))
		
			#prepare output values
			res = result
			usize = len(url)
			dsize = len(lines)
			
		except Exception, e:
			#unable to connect to foxy server
			Log.log('CONNECT : unable to connect to foxy server [%s]' % str(e))
			res = False
			
		return res,usize,dsize
			
	##Parse connection results
	def __parseConnectionResults(self, lines):
		#first line needs to be "MSG"
		if lines[0]=='MSG':
			#ok, continu
			if lines[1]==self.FOXYTAG_OK_INIT:
				return (True, self.FOXYTAG_OK_INIT)
			elif lines[1]==self.FOXYTAG_KO:
				return (False, self.FOXYTAG_KO)
			elif lines[1]==self.FOXYTAG_KO_CMD:
				return (False, self.FOXYTAG_KO_CMD)
			elif lines[1]==self.FOXYTAG_KO_VERSION:
				return (False, self.FOXYTAG_KO_VERSION)
			elif lines[1]==self.FOXYTAG_KO_LOGIN:
				return (False, self.FOXYTAG_KO_LOGIN)
			elif lines[1].startswith(self.FOXYTAG_ERR):
				return (False, (lines[1][3:len(lines[1])]).strip())
			elif lines[1].startswith(self.FOXYTAG_INF):
				return (False, (lines[1][3:len(lines[1])]).strip())
			elif lines[1].startswith(self.FOXYTAG_PUB):
				return (False, (lines[1][3:len(lines[1])]).strip())
			else:
				return (False, self.FOXYTAG_ERR_UNKNOWN)
		else:
			return (False, self.FOXYTAG_ERR_UNEXPECTED)
	
	##Get speedcams infos
	##@param lat : latitude
	##@param long : longitude
	##@param radius : radius (in meters, default 7250 meters)
	##@return res : result of tag post
	##@return usize : size of uploaded data
	##@return dsize : size of downloaded data
	def tagRequest(self, lat, long, radius=7250):
		#init
		res = False
		usize = 0
		dsize = 0
		
		#prepare data to post
		data = urllib.urlencode({	'cmd' : 'tagrequest', 
									'username' : self.__FOXYTAG_USERNAME,
									'password' : self.__FOXYTAG_PASSWORD,
									'client' : self.__FOXYTAG_CLIENT,
									'lat' : lat, 
									'lon' : long, 
									'radius' : radius})

		try:
			#get form
			url = self.__FOXYTAG_HOST+'?%s' % data
			handler = urllib.urlopen(url)
			lines = handler.read()
			lines = lines.split('\n')
			
			#parse results
			(result, message) = self.__parseConnectionResults(lines)
			Log.log('TAGREQUEST : result=%s message=%s' % (result, message))
			if not result:
				#no result with connection parsing, try to parse tags
				(result, message) = self.__parseTagRequestResults(lines)
				Log.log('TAGREQUEST : result=%s message=%s' % (result, message))
				if result:
					#display tags (debug)
					for i in range(len(self.__tags)):
						tag = self.__tags[i];
						Log.log('lat=%.5f long=%.5f' % (tag.lat, tag.long))
					
					#prepare outputs
					res = True
					usize = len(url)
					dsize = len(lines)
					
				else:
					#unable to request tags
					res = False
			else:
				#connection result found, unable to request tags
				Log.log('TAGREQUEST : unable to connect to foxy server')
				res = False
		except:
			#unable to connect to foxy server
			res = False
			
		return res,usize,dsize

	##Parse tag request results
	def __parseTagRequestResults(self, lines):
		#first line needs to be "TAG"
		if lines[0]=='TAG':
			#ok, clear current tags
			del self.__tags[0:len(self.__tags)]
			#second line is tag request, go ahead
			for i in range(2,len(lines)):
				if len(lines[i])>0:
					self.__tags.append( FoxyTag(lines[i]) )
			return (True, '')
		else:
			return (False, self.FOXYTAG_ERR_UNEXPECTED)
			
	##Get tags
	def getTags(self):
		return self.__tags
			
	##Send tag to server
	##@return res : result of tag post
	##@return usize : size of uploaded data
	##@return dsize : size of downloaded data
	def tagPost(self, tag, speed):
		#init
		res = False
		usize = 0
		dsize = 0
		
		if tag!=None and speed>20 and speed<1000:
			#prepare data to post
			data = urllib.urlencode({'cmd':'tagpost', 'username':self.__FOXYTAG_USERNAME, 'password':self.__FOXYTAG_PASSWORD, 'client':self.__FOXYTAG_CLIENT, 'lat':tag.lat, 'lon':tag.long, 'kind':tag.kind, 'heading':tag.heading, 'speed':speed})
		
			try:
				#get form
				url = self.__FOXYTAG_HOST+'?%s' % data
				Log.log('post string = '+ url)
				#handler = urllib.urlopen(url)
				#lines = handler.read()
				#lines = lines.split('\n')
			
				#parse results
				#(result, message) = self.__parseTagPostResults(lines)
				#Log.log('TAGPOST : result=%s message=%s' % (result, message))
			
				res = True
				usize = len(url)
				dsize = len(lines)
			except:
				#unable to connect to foxy server
				Log.log('TAGPOST : unable to connect to foxy server')
				res = False
		else:
			#no tag specified
			res = False
			
		return res, usize, dsize
			
	##Parse connection results
	def __parseTagPostResults(self, lines):
		#first line needs to be "MSG"
		if lines[0]=='MSG':
			#ok, continu
			if lines[1]==self.FOXYTAG_OK:
				return (True, self.FOXYTAG_OK)
			elif lines[1]==self.FOXYTAG_KO:
				return (False, self.FOXYTAG_KO)
			elif lines[1]==self.FOXYTAG_KO_CMD:
				return (False, self.FOXYTAG_KO_CMD)
			elif lines[1]==self.FOXYTAG_KO_LOGIN:
				return (False, self.FOXYTAG_KO_LOGIN)
			elif lines[1].startswith(self.FOXYTAG_ERR):
				return (False, (lines[1][3:len(lines[1])]).strip())
			elif lines[1].startswith(self.FOXYTAG_INF):
				return (False, (lines[1][3:len(lines[1])]).strip())
			else:
				return (False, self.FOXYTAG_ERR_UNKNOWN)
		else:
			return (False, self.FOXYTAG_ERR_UNEXPECTED)
			